@@ -208,7 +208,7 @@ int ssh_gssapi_handle_userauth(ssh_session session, const char *user, uint32_t n
                 return SSH_ERROR;
             session->gssapi->state = SSH_GSSAPI_STATE_RCV_TOKEN;
             rc = ssh_gssapi_send_response(session, oid_s);
-            ssh_string_free(oid_s);
+            SSH_STRING_FREE(oid_s);
             return rc;
         } else {
             return ssh_auth_reply_default(session,0);
@@ -235,6 +235,10 @@ int ssh_gssapi_handle_userauth(ssh_session session, const char *user, uint32_t n
     for (i=0 ; i< n_oid ; ++i){
         unsigned char *oid_s = (unsigned char *) ssh_string_data(oids[i]);
         size_t len = ssh_string_len(oids[i]);
+
+        if (oid_s == NULL) {
+            continue;
+        }
         if(len < 2 || oid_s[0] != SSH_OID_TAG || ((size_t)oid_s[1]) != len - 2){
             SSH_LOG(SSH_LOG_WARNING,"GSSAPI: received invalid OID");
             continue;
@@ -293,6 +297,10 @@ int ssh_gssapi_handle_userauth(ssh_session session, const char *user, uint32_t n
     for (i=0 ; i< n_oid ; ++i){
         unsigned char *oid_s = (unsigned char *) ssh_string_data(oids[i]);
         size_t len = ssh_string_len(oids[i]);
+
+        if (oid_s == NULL) {
+            continue;
+        }
         if(len < 2 || oid_s[0] != SSH_OID_TAG || ((size_t)oid_s[1]) != len - 2){
             SSH_LOG(SSH_LOG_WARNING,"GSSAPI: received invalid OID");
             continue;
@@ -384,7 +392,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_server){
                 return SSH_PACKET_USED;
             }
             ssh_packet_send(session);
-            ssh_string_free(out_token);
+            SSH_STRING_FREE(out_token);
         } else {
             session->gssapi->state = SSH_GSSAPI_STATE_RCV_MIC;
         }
@@ -403,7 +411,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_server){
                          "accepting token",
                          maj_stat,
                          min_stat);
-    ssh_string_free(token);
+    SSH_STRING_FREE(token);
     if (client_name != GSS_C_NO_NAME){
         session->gssapi->client_name = client_name;
         session->gssapi->canonic_user = ssh_gssapi_name_to_char(client_name);
@@ -444,13 +452,17 @@ static ssh_buffer ssh_gssapi_build_mic(ssh_session session)
     ssh_buffer mic_buffer = NULL;
     int rc;
 
+    crypto = ssh_packet_get_current_crypto(session, SSH_DIRECTION_BOTH);
+    if (crypto == NULL) {
+        return NULL;
+    }
+
     mic_buffer = ssh_buffer_new();
     if (mic_buffer == NULL) {
         ssh_set_error_oom(session);
         return NULL;
     }
 
-    crypto = ssh_packet_get_current_crypto(session, SSH_DIRECTION_BOTH);
     rc = ssh_buffer_pack(mic_buffer,
                          "dPbsss",
                          crypto->digest_len,
@@ -461,7 +473,7 @@ static ssh_buffer ssh_gssapi_build_mic(ssh_session session)
                          "gssapi-with-mic");
     if (rc != SSH_OK) {
         ssh_set_error_oom(session);
-        ssh_buffer_free(mic_buffer);
+        SSH_BUFFER_FREE(mic_buffer);
         return NULL;
     }
 
@@ -545,10 +557,10 @@ error:
 end:
     ssh_gssapi_free(session);
     if (mic_buffer != NULL) {
-        ssh_buffer_free(mic_buffer);
+        SSH_BUFFER_FREE(mic_buffer);
     }
     if (mic_token != NULL) {
-        ssh_string_free(mic_token);
+        SSH_STRING_FREE(mic_token);
     }
 
     return SSH_PACKET_USED;
@@ -700,13 +712,13 @@ end:
  *                            later.
  */
 int ssh_gssapi_auth_mic(ssh_session session){
-    int i;
+    size_t i;
     gss_OID_set selected; /* oid selected for authentication */
-    ssh_string *oids;
+    ssh_string *oids = NULL;
     int rc;
-    int n_oids = 0;
+    size_t n_oids = 0;
     OM_uint32 maj_stat, min_stat;
-    char name_buf[256];
+    char name_buf[256] = {0};
     gss_buffer_desc hostname;
     const char *gss_host = session->opts.host;
 
@@ -750,7 +762,7 @@ int ssh_gssapi_auth_mic(ssh_session session){
     }
 
     n_oids = selected->count;
-    SSH_LOG(SSH_LOG_PROTOCOL, "Sending %d oids", n_oids);
+    SSH_LOG(SSH_LOG_PROTOCOL, "Sending %zu oids", n_oids);
 
     oids = calloc(n_oids, sizeof(ssh_string));
     if (oids == NULL) {
@@ -760,6 +772,11 @@ int ssh_gssapi_auth_mic(ssh_session session){
 
     for (i=0; i<n_oids; ++i){
         oids[i] = ssh_string_new(selected->elements[i].length + 2);
+        if (oids[i] == NULL) {
+            ssh_set_error_oom(session);
+            rc = SSH_ERROR;
+            goto out;
+        }
         ((unsigned char *)oids[i]->data)[0] = SSH_OID_TAG;
         ((unsigned char *)oids[i]->data)[1] = selected->elements[i].length;
         memcpy((unsigned char *)oids[i]->data + 2, selected->elements[i].elements,
@@ -767,8 +784,10 @@ int ssh_gssapi_auth_mic(ssh_session session){
     }
 
     rc = ssh_gssapi_send_auth_mic(session, oids, n_oids);
+
+out:
     for (i = 0; i < n_oids; i++) {
-        ssh_string_free(oids[i]);
+        SSH_STRING_FREE(oids[i]);
     }
     free(oids);
     if (rc != SSH_ERROR) {
@@ -778,13 +797,13 @@ int ssh_gssapi_auth_mic(ssh_session session){
     return SSH_AUTH_ERROR;
 }
 
-static gss_OID ssh_gssapi_oid_from_string(ssh_string oid_s){
-    gss_OID ret;
+static gss_OID ssh_gssapi_oid_from_string(ssh_string oid_s)
+{
+    gss_OID ret = NULL;
     unsigned char *data = ssh_string_data(oid_s);
     size_t len = ssh_string_len(oid_s);
 
-    ret = malloc(sizeof(gss_OID_desc));
-    if (ret == NULL) {
+    if (data == NULL) {
         return NULL;
     }
 
@@ -792,10 +811,17 @@ static gss_OID ssh_gssapi_oid_from_string(ssh_string oid_s){
         SAFE_FREE(ret);
         return NULL;
     }
+
     if (data[0] != SSH_OID_TAG || data[1] != len - 2) {
         SAFE_FREE(ret);
         return NULL;
     }
+
+    ret = malloc(sizeof(gss_OID_desc));
+    if (ret == NULL) {
+        return NULL;
+    }
+
     ret->elements = malloc(len - 2);
     if (ret->elements == NULL) {
         SAFE_FREE(ret);
@@ -828,7 +854,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_response){
         goto error;
     }
     session->gssapi->client.oid = ssh_gssapi_oid_from_string(oid_s);
-    ssh_string_free(oid_s);
+    SSH_STRING_FREE(oid_s);
     if (!session->gssapi->client.oid) {
         ssh_set_error(session, SSH_FATAL, "Invalid OID");
         goto error;
@@ -896,7 +922,7 @@ static int ssh_gssapi_send_mic(ssh_session session){
     maj_stat = gss_get_mic(&min_stat,session->gssapi->ctx, GSS_C_QOP_DEFAULT,
                            &mic_buf, &mic_token_buf);
     if (GSS_ERROR(maj_stat)){
-        ssh_buffer_free(mic_buffer);
+        SSH_BUFFER_FREE(mic_buffer);
         ssh_gssapi_log_error(SSH_LOG_PROTOCOL,
                              "generating MIC",
                              maj_stat,
@@ -910,7 +936,7 @@ static int ssh_gssapi_send_mic(ssh_session session){
                          mic_token_buf.length,
                          (size_t)mic_token_buf.length, mic_token_buf.value);
     if (rc != SSH_OK) {
-        ssh_buffer_free(mic_buffer);
+        SSH_BUFFER_FREE(mic_buffer);
         ssh_set_error_oom(session);
         return SSH_ERROR;
     }
@@ -958,7 +984,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_client){
                          "accepting token",
                          maj_stat,
                          min_stat);
-    ssh_string_free(token);
+    SSH_STRING_FREE(token);
     if (GSS_ERROR(maj_stat)){
         ssh_gssapi_log_error(SSH_LOG_PROTOCOL,
                              "Gssapi error",
