@@ -143,7 +143,7 @@ void libssh_constructor(void)
  * If the library is already initialized, increments the _ssh_initialized
  * counter and return the error code cached in _ssh_init_ret.
  *
- * @returns             0 on success, -1 if an error occured.
+ * @returns             SSH_OK on success, SSH_ERROR if an error occurred.
  */
 int ssh_init(void) {
     return _ssh_init(0);
@@ -188,7 +188,7 @@ _ret:
  *
  * This function is automatically called when the library is unloaded.
  *
- * @returns             0 on succes, -1 if an error occured.
+ * @returns             SSH_OK on success, SSH_ERROR if an error occurred.
  *
  */
 void libssh_destructor(void)
@@ -224,24 +224,32 @@ int ssh_finalize(void) {
 
 #ifdef _WIN32
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && !defined(LIBSSH_STATIC)
 /* Library constructor and destructor */
-BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+BOOL WINAPI DllMain(HINSTANCE hinstDLL,
+                    DWORD fdwReason,
+                    LPVOID lpvReserved)
 {
-    int rc;
+    int rc = 0;
 
-    if (fdwReason == DLL_PROCESS_ATTACH) {
-        rc = ssh_init();
-    } else if (fdwReason == DLL_PROCESS_DETACH) {
-        rc = ssh_finalize();
+    switch(fdwReason) {
+    case DLL_PROCESS_ATTACH:
+        rc = _ssh_init(1);
+        if (rc != 0) {
+            fprintf(stderr, "DllMain: ssh_init failed!");
+            return FALSE;
+        }
+        break;
+    case DLL_PROCESS_DETACH:
+        _ssh_finalize(1);
+        break;
+    default:
+        break;
     }
 
-    if (rc != 0) {
-        return FALSE;
-    }
     return TRUE;
 }
-#endif /* _MSC_VER */
+#endif /* _MSC_VER && !LIBSSH_STATIC */
 
 #endif /* _WIN32 */
 
