@@ -26,17 +26,17 @@
 #include "libssh/wrapper.h"
 #include "libssh/crypto.h"
 #include "libssh/priv.h"
+#include "libssh/misc.h"
 
 #ifdef HAVE_LIBMBEDCRYPTO
 #include <mbedtls/md.h>
+#ifdef MBEDTLS_GCM_C
+#include <mbedtls/gcm.h>
+#endif /* MBEDTLS_GCM_C */
 
 static mbedtls_entropy_context ssh_mbedtls_entropy;
 static mbedtls_ctr_drbg_context ssh_mbedtls_ctr_drbg;
 
-struct ssh_mac_ctx_struct {
-    enum ssh_mac_e mac_type;
-    mbedtls_md_context_t ctx;
-};
 static int libmbedcrypto_initialized = 0;
 
 void ssh_reseed(void)
@@ -94,7 +94,7 @@ void sha1_final(unsigned char *md, SHACTX c)
     SAFE_FREE(c);
 }
 
-void sha1(unsigned char *digest, int len, unsigned char *hash)
+void sha1(const unsigned char *digest, int len, unsigned char *hash)
 {
     const mbedtls_md_info_t *md_info =
         mbedtls_md_info_from_type(MBEDTLS_MD_SHA1);
@@ -222,7 +222,7 @@ void sha256_final(unsigned char *md, SHA256CTX c)
     SAFE_FREE(c);
 }
 
-void sha256(unsigned char *digest, int len, unsigned char *hash)
+void sha256(const unsigned char *digest, int len, unsigned char *hash)
 {
     const mbedtls_md_info_t *md_info =
         mbedtls_md_info_from_type(MBEDTLS_MD_SHA256);
@@ -276,7 +276,7 @@ void sha384_final(unsigned char *md, SHA384CTX c)
     SAFE_FREE(c);
 }
 
-void sha384(unsigned char *digest, int len, unsigned char *hash)
+void sha384(const unsigned char *digest, int len, unsigned char *hash)
 {
     const mbedtls_md_info_t *md_info =
         mbedtls_md_info_from_type(MBEDTLS_MD_SHA384);
@@ -329,7 +329,7 @@ void sha512_final(unsigned char *md, SHA512CTX c)
     SAFE_FREE(c);
 }
 
-void sha512(unsigned char *digest, int len, unsigned char *hash)
+void sha512(const unsigned char *digest, int len, unsigned char *hash)
 {
     const mbedtls_md_info_t *md_info =
         mbedtls_md_info_from_type(MBEDTLS_MD_SHA512);
@@ -382,66 +382,13 @@ void md5_final(unsigned char *md, MD5CTX c)
     SAFE_FREE(c);
 }
 
-ssh_mac_ctx ssh_mac_ctx_init(enum ssh_mac_e type)
+int ssh_kdf(struct ssh_crypto_struct *crypto,
+            unsigned char *key, size_t key_len,
+            int key_type, unsigned char *output,
+            size_t requested_len)
 {
-    ssh_mac_ctx ctx = malloc(sizeof (struct ssh_mac_ctx_struct));
-    const mbedtls_md_info_t *md_info;
-    int rc;
-    if (ctx == NULL) {
-        return NULL;
-    }
-
-    ctx->mac_type=type;
-    switch(type) {
-        case SSH_MAC_SHA1:
-            md_info = mbedtls_md_info_from_type(MBEDTLS_MD_SHA1);
-            break;
-        case SSH_MAC_SHA256:
-            md_info = mbedtls_md_info_from_type(MBEDTLS_MD_SHA256);
-            break;
-        case SSH_MAC_SHA384:
-            md_info = mbedtls_md_info_from_type(MBEDTLS_MD_SHA384);
-            break;
-        case SSH_MAC_SHA512:
-            md_info = mbedtls_md_info_from_type(MBEDTLS_MD_SHA512);
-            break;
-        default:
-            goto error;
-    }
-
-    if (md_info == NULL) {
-        goto error;
-    }
-
-    mbedtls_md_init(&ctx->ctx);
-
-    rc = mbedtls_md_setup(&ctx->ctx, md_info, 0);
-    if (rc != 0) {
-        goto error;
-    }
-
-    rc = mbedtls_md_starts(&ctx->ctx);
-    if (rc != 0) {
-        goto error;
-    }
-
-    return ctx;
-
-error:
-    SAFE_FREE(ctx);
-    return NULL;
-}
-
-void ssh_mac_update(ssh_mac_ctx ctx, const void *data, unsigned long len)
-{
-    mbedtls_md_update(&ctx->ctx, data, len);
-}
-
-void ssh_mac_final(unsigned char *md, ssh_mac_ctx ctx)
-{
-    mbedtls_md_finish(&ctx->ctx, md);
-    mbedtls_md_free(&ctx->ctx);
-    SAFE_FREE(ctx);
+    return sshkdf_derive_key(crypto, key, key_len,
+                             key_type, output, requested_len);
 }
 
 HMACCTX hmac_init(const void *key, int len, enum ssh_hmac_e type)
@@ -506,39 +453,68 @@ void hmac_final(HMACCTX c, unsigned char *hashmacbuf, unsigned int *len)
     SAFE_FREE(c);
 }
 
-static int cipher_set_encrypt_key(struct ssh_cipher_struct *cipher, void *key,
-        void *IV)
+static int
+cipher_init(struct ssh_cipher_struct *cipher,
+            mbedtls_operation_t operation,
+            void *key,
+            void *IV)
 {
-
     const mbedtls_cipher_info_t *cipher_info = NULL;
+    mbedtls_cipher_context_t *ctx;
     int rc;
 
-    mbedtls_cipher_init(&cipher->encrypt_ctx);
+    if (operation == MBEDTLS_ENCRYPT) {
+        ctx = &cipher->encrypt_ctx;
+    } else if (operation == MBEDTLS_DECRYPT) {
+        ctx = &cipher->decrypt_ctx;
+    } else {
+        SSH_LOG(SSH_LOG_WARNING, "unknown operation");
+        return 1;
+    }
+
+    mbedtls_cipher_init(ctx);
     cipher_info = mbedtls_cipher_info_from_type(cipher->type);
 
-    rc = mbedtls_cipher_setup(&cipher->encrypt_ctx, cipher_info);
+    rc = mbedtls_cipher_setup(ctx, cipher_info);
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setup failed");
         goto error;
     }
 
-    rc = mbedtls_cipher_setkey(&cipher->encrypt_ctx, key,
+    rc = mbedtls_cipher_setkey(ctx, key,
                                cipher_info->key_bitlen,
-                               MBEDTLS_ENCRYPT);
+                               operation);
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setkey failed");
         goto error;
     }
 
-    rc = mbedtls_cipher_set_iv(&cipher->encrypt_ctx, IV, cipher_info->iv_size);
-
+    rc = mbedtls_cipher_set_iv(ctx, IV, cipher_info->iv_size);
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_set_iv failed");
         goto error;
     }
 
-    rc = mbedtls_cipher_reset(&cipher->encrypt_ctx);
+    return 0;
+error:
+    mbedtls_cipher_free(ctx);
+    return 1;
+}
 
+static int
+cipher_set_encrypt_key(struct ssh_cipher_struct *cipher,
+                       void *key,
+                       void *IV)
+{
+    int rc;
+
+    rc = cipher_init(cipher, MBEDTLS_ENCRYPT, key, IV);
+    if (rc != 0) {
+        SSH_LOG(SSH_LOG_WARNING, "cipher_init failed");
+        goto error;
+    }
+
+    rc = mbedtls_cipher_reset(&cipher->encrypt_ctx);
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_reset failed");
         goto error;
@@ -546,38 +522,19 @@ static int cipher_set_encrypt_key(struct ssh_cipher_struct *cipher, void *key,
 
     return SSH_OK;
 error:
-    mbedtls_cipher_free(&cipher->encrypt_ctx);
     return SSH_ERROR;
 }
 
-static int cipher_set_encrypt_key_cbc(struct ssh_cipher_struct *cipher, void *key,
-        void *IV)
+static int
+cipher_set_encrypt_key_cbc(struct ssh_cipher_struct *cipher,
+                           void *key,
+                           void *IV)
 {
-
-    const mbedtls_cipher_info_t *cipher_info = NULL;
     int rc;
 
-    mbedtls_cipher_init(&cipher->encrypt_ctx);
-    cipher_info = mbedtls_cipher_info_from_type(cipher->type);
-
-    rc = mbedtls_cipher_setup(&cipher->encrypt_ctx, cipher_info);
+    rc = cipher_init(cipher, MBEDTLS_ENCRYPT, key, IV);
     if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setup failed");
-        goto error;
-    }
-
-    rc = mbedtls_cipher_setkey(&cipher->encrypt_ctx, key,
-                               cipher_info->key_bitlen,
-                               MBEDTLS_ENCRYPT);
-    if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setkey failed");
-        goto error;
-    }
-
-    rc = mbedtls_cipher_set_iv(&cipher->encrypt_ctx, IV, cipher_info->iv_size);
-
-    if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_set_iv failed");
+        SSH_LOG(SSH_LOG_WARNING, "cipher_init failed");
         goto error;
     }
 
@@ -592,7 +549,6 @@ static int cipher_set_encrypt_key_cbc(struct ssh_cipher_struct *cipher, void *ke
     }
 
     rc = mbedtls_cipher_reset(&cipher->encrypt_ctx);
-
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_reset failed");
         goto error;
@@ -604,37 +560,51 @@ error:
     return SSH_ERROR;
 }
 
-static int cipher_set_decrypt_key(struct ssh_cipher_struct *cipher, void *key,
-        void *IV)
+#ifdef MBEDTLS_GCM_C
+static int
+cipher_set_key_gcm(struct ssh_cipher_struct *cipher,
+                   void *key,
+                   void *IV)
 {
     const mbedtls_cipher_info_t *cipher_info = NULL;
     int rc;
 
-    mbedtls_cipher_init(&cipher->decrypt_ctx);
+    mbedtls_gcm_init(&cipher->gcm_ctx);
     cipher_info = mbedtls_cipher_info_from_type(cipher->type);
 
-    rc = mbedtls_cipher_setup(&cipher->decrypt_ctx, cipher_info);
+    rc = mbedtls_gcm_setkey(&cipher->gcm_ctx,
+                            MBEDTLS_CIPHER_ID_AES,
+                            key,
+                            cipher_info->key_bitlen);
     if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setkey failed");
+        SSH_LOG(SSH_LOG_WARNING, "mbedtls_gcm_setkey failed");
         goto error;
     }
 
-    rc = mbedtls_cipher_setkey(&cipher->decrypt_ctx, key,
-                               cipher_info->key_bitlen,
-                               MBEDTLS_DECRYPT);
-    if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setkey failed");
-        goto error;
-    }
+    /* Store the IV so we can increment the packet counter later */
+    memcpy(cipher->last_iv, IV, AES_GCM_IVLEN);
 
-    rc = mbedtls_cipher_set_iv(&cipher->decrypt_ctx, IV, cipher_info->iv_size);
+    return 0;
+error:
+    mbedtls_gcm_free(&cipher->gcm_ctx);
+    return 1;
+}
+#endif /* MBEDTLS_GCM_C */
+
+static int
+cipher_set_decrypt_key(struct ssh_cipher_struct *cipher,
+                       void *key,
+                       void *IV)
+{
+    int rc;
+
+    rc = cipher_init(cipher, MBEDTLS_DECRYPT, key, IV);
     if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_set_iv failed");
+        SSH_LOG(SSH_LOG_WARNING, "cipher_init failed");
         goto error;
     }
 
     mbedtls_cipher_reset(&cipher->decrypt_ctx);
-
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_reset failed");
         goto error;
@@ -646,45 +616,27 @@ error:
     return SSH_ERROR;
 }
 
-static int cipher_set_decrypt_key_cbc(struct ssh_cipher_struct *cipher, void *key,
-        void *IV)
+static int
+cipher_set_decrypt_key_cbc(struct ssh_cipher_struct *cipher,
+                           void *key,
+                           void *IV)
 {
-    const mbedtls_cipher_info_t *cipher_info;
     int rc;
 
-    mbedtls_cipher_init(&cipher->decrypt_ctx);
-    cipher_info = mbedtls_cipher_info_from_type(cipher->type);
-
-    rc = mbedtls_cipher_setup(&cipher->decrypt_ctx, cipher_info);
+    rc = cipher_init(cipher, MBEDTLS_DECRYPT, key, IV);
     if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setkey failed");
-        goto error;
-    }
-
-    rc = mbedtls_cipher_setkey(&cipher->decrypt_ctx, key,
-                               cipher_info->key_bitlen,
-                               MBEDTLS_DECRYPT);
-    if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_setkey failed");
-        goto error;
-    }
-
-    rc = mbedtls_cipher_set_iv(&cipher->decrypt_ctx, IV, cipher_info->iv_size);
-    if (rc != 0) {
-        SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_set_iv failed");
+        SSH_LOG(SSH_LOG_WARNING, "cipher_init failed");
         goto error;
     }
 
     rc = mbedtls_cipher_set_padding_mode(&cipher->decrypt_ctx,
             MBEDTLS_PADDING_NONE);
-
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_set_padding_mode failed");
         goto error;
     }
 
     mbedtls_cipher_reset(&cipher->decrypt_ctx);
-
     if (rc != 0) {
         SSH_LOG(SSH_LOG_WARNING, "mbedtls_cipher_reset failed");
         goto error;
@@ -696,8 +648,10 @@ error:
     return SSH_ERROR;
 }
 
-static void cipher_encrypt(struct ssh_cipher_struct *cipher, void *in, void *out,
-        unsigned long len)
+static void cipher_encrypt(struct ssh_cipher_struct *cipher,
+                           void *in,
+                           void *out,
+                           size_t len)
 {
     size_t outlen = 0;
     size_t total_len = 0;
@@ -751,8 +705,10 @@ static void cipher_encrypt_cbc(struct ssh_cipher_struct *cipher, void *in, void
 
 }
 
-static void cipher_decrypt(struct ssh_cipher_struct *cipher, void *in, void *out,
-        unsigned long len)
+static void cipher_decrypt(struct ssh_cipher_struct *cipher,
+                           void *in,
+                           void *out,
+                           size_t len)
 {
     size_t outlen = 0;
     int rc = 0;
@@ -833,13 +789,109 @@ static void cipher_decrypt_cbc(struct ssh_cipher_struct *cipher, void *in, void
 
 }
 
+#ifdef MBEDTLS_GCM_C
+static int
+cipher_gcm_get_length(struct ssh_cipher_struct *cipher,
+                      void *in,
+                      uint8_t *out,
+                      size_t len,
+                      uint64_t seq)
+{
+    (void)cipher;
+    (void)seq;
+
+    /* The length is not encrypted: Copy it to the result buffer */
+    memcpy(out, in, len);
+
+    return SSH_OK;
+}
+
+static void
+cipher_encrypt_gcm(struct ssh_cipher_struct *cipher,
+                   void *in,
+                   void *out,
+                   size_t len,
+                   uint8_t *tag,
+                   uint64_t seq)
+{
+    size_t authlen, aadlen;
+    int rc;
+
+    (void) seq;
+
+    aadlen = cipher->lenfield_blocksize;
+    authlen = cipher->tag_size;
+
+    /* The length is not encrypted */
+    memcpy(out, in, aadlen);
+    rc = mbedtls_gcm_crypt_and_tag(&cipher->gcm_ctx,
+                                   MBEDTLS_GCM_ENCRYPT,
+                                   len - aadlen, /* encrypted data len */
+                                   cipher->last_iv, /* IV */
+                                   AES_GCM_IVLEN,
+                                   in, /* aad */
+                                   aadlen,
+                                   (const unsigned char *)in + aadlen, /* input */
+                                   (unsigned char *)out + aadlen, /* output */
+                                   authlen,
+                                   tag); /* tag */
+    if (rc != 0) {
+        SSH_LOG(SSH_LOG_WARNING, "mbedtls_gcm_crypt_and_tag failed");
+        return;
+    }
+
+    /* Increment the IV for the next invocation */
+    uint64_inc(cipher->last_iv + 4);
+}
+
+static int
+cipher_decrypt_gcm(struct ssh_cipher_struct *cipher,
+                   void *complete_packet,
+                   uint8_t *out,
+                   size_t encrypted_size,
+                   uint64_t seq)
+{
+    size_t authlen, aadlen;
+    int rc;
+
+    (void) seq;
+
+    aadlen = cipher->lenfield_blocksize;
+    authlen = cipher->tag_size;
+
+    rc = mbedtls_gcm_auth_decrypt(&cipher->gcm_ctx,
+                                  encrypted_size, /* encrypted data len */
+                                  cipher->last_iv, /* IV */
+                                  AES_GCM_IVLEN,
+                                  complete_packet, /* aad */
+                                  aadlen,
+                                  (const uint8_t *)complete_packet + aadlen + encrypted_size, /* tag */
+                                  authlen,
+                                  (const uint8_t *)complete_packet + aadlen, /* input */
+                                  (unsigned char *)out); /* output */
+    if (rc != 0) {
+        SSH_LOG(SSH_LOG_WARNING, "mbedtls_gcm_auth_decrypt failed");
+        return SSH_ERROR;
+    }
+
+    /* Increment the IV for the next invocation */
+    uint64_inc(cipher->last_iv + 4);
+
+    return SSH_OK;
+}
+#endif /* MBEDTLS_GCM_C */
+
 static void cipher_cleanup(struct ssh_cipher_struct *cipher)
 {
     mbedtls_cipher_free(&cipher->encrypt_ctx);
     mbedtls_cipher_free(&cipher->decrypt_ctx);
+#ifdef MBEDTLS_GCM_C
+    mbedtls_gcm_free(&cipher->gcm_ctx);
+#endif /* MBEDTLS_GCM_C */
 }
 
 static struct ssh_cipher_struct ssh_ciphertab[] = {
+#ifdef WITH_BLOWFISH_CIPHER
     {
         .name = "blowfish-cbc",
         .blocksize = 8,
@@ -851,6 +903,7 @@ static struct ssh_cipher_struct ssh_ciphertab[] = {
         .decrypt = cipher_decrypt_cbc,
         .cleanup = cipher_cleanup
     },
+#endif /* WITH_BLOWFISH_CIPHER */
     {
         .name = "aes128-ctr",
         .blocksize = 16,
@@ -917,6 +970,36 @@ static struct ssh_cipher_struct ssh_ciphertab[] = {
         .decrypt = cipher_decrypt_cbc,
         .cleanup = cipher_cleanup
     },
+#ifdef MBEDTLS_GCM_C
+    {
+        .name = "aes128-gcm@openssh.com",
+        .blocksize = 16,
+        .lenfield_blocksize = 4, /* not encrypted, but authenticated */
+        .keysize = 128,
+        .tag_size = AES_GCM_TAGLEN,
+        .type = MBEDTLS_CIPHER_AES_128_GCM,
+        .set_encrypt_key = cipher_set_key_gcm,
+        .set_decrypt_key = cipher_set_key_gcm,
+        .aead_encrypt = cipher_encrypt_gcm,
+        .aead_decrypt_length = cipher_gcm_get_length,
+        .aead_decrypt = cipher_decrypt_gcm,
+        .cleanup = cipher_cleanup
+    },
+    {
+        .name = "aes256-gcm@openssh.com",
+        .blocksize = 16,
+        .lenfield_blocksize = 4, /* not encrypted, but authenticated */
+        .keysize = 256,
+        .tag_size = AES_GCM_TAGLEN,
+        .type = MBEDTLS_CIPHER_AES_256_GCM,
+        .set_encrypt_key = cipher_set_key_gcm,
+        .set_decrypt_key = cipher_set_key_gcm,
+        .aead_encrypt = cipher_encrypt_gcm,
+        .aead_decrypt_length = cipher_gcm_get_length,
+        .aead_decrypt = cipher_decrypt_gcm,
+        .cleanup = cipher_cleanup
+    },
+#endif /* MBEDTLS_GCM_C */
     {
         .name = "3des-cbc",
         .blocksize = 8,
