@@ -139,7 +139,7 @@ static ssh_string asn1_get(ssh_buffer buffer, unsigned char want) {
   }
 
   if (ssh_buffer_get_data(buffer, ssh_string_data(str), size) == 0) {
-    ssh_string_free(str);
+    SSH_STRING_FREE(str);
     return NULL;
   }
 
@@ -177,14 +177,14 @@ static ssh_string asn1_get_bit_string(ssh_buffer buffer)
 
     len = ssh_buffer_get_data(buffer, &unused, 1);
     if (len == 0) {
-        ssh_string_free(str);
+        SSH_STRING_FREE(str);
         return NULL;
     }
 
     if (unused == 0) {
         len = ssh_buffer_get_data(buffer, ssh_string_data(str), size);
         if (len == 0) {
-            ssh_string_free(str);
+            SSH_STRING_FREE(str);
             return NULL;
         }
         return str;
@@ -197,7 +197,7 @@ static ssh_string asn1_get_bit_string(ssh_buffer buffer)
 
         len = ssh_buffer_get_data(buffer, &c, 1);
         if (len == 0) {
-            ssh_string_free(str);
+            SSH_STRING_FREE(str);
             return NULL;
         }
         *p = last | (c >> unused);
@@ -434,7 +434,7 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
             header_end = ECDSA_HEADER_END;
             break;
         default:
-            ssh_buffer_free(buffer);
+            SSH_BUFFER_FREE(buffer);
             return NULL;
     }
 
@@ -450,14 +450,14 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
         get_next_line(p, len);
     }
     if (eol) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         return NULL;
     }
 
     /* skip header line */
     get_next_line(p, len);
     if (eol) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         return NULL;
     }
 
@@ -465,7 +465,7 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
         /* skip line */
         get_next_line(p, len);
         if (eol) {
-            ssh_buffer_free(buffer);
+            SSH_BUFFER_FREE(buffer);
             return NULL;
         }
 
@@ -474,24 +474,24 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
             len = 0;
             get_next_line(p, len);
             if (eol) {
-                ssh_buffer_free(buffer);
+                SSH_BUFFER_FREE(buffer);
                 return NULL;
             }
             if (privatekey_dek_header(p, len, &algo, &mode, &key_len,
                         &iv, &iv_len) < 0) {
-                ssh_buffer_free(buffer);
+                SSH_BUFFER_FREE(buffer);
                 SAFE_FREE(iv);
                 return NULL;
             }
         } else {
-            ssh_buffer_free(buffer);
+            SSH_BUFFER_FREE(buffer);
             SAFE_FREE(iv);
             return NULL;
         }
     } else {
         if(len > 0) {
             if (ssh_buffer_add_data(buffer, p, len) < 0) {
-                ssh_buffer_free(buffer);
+                SSH_BUFFER_FREE(buffer);
                 SAFE_FREE(iv);
                 return NULL;
             }
@@ -501,7 +501,7 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
     get_next_line(p, len);
     while(!eol && strncmp(p, header_end, header_end_size) != 0) {
         if (ssh_buffer_add_data(buffer, p, len) < 0) {
-            ssh_buffer_free(buffer);
+            SSH_BUFFER_FREE(buffer);
             SAFE_FREE(iv);
             return NULL;
         }
@@ -509,19 +509,19 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
     }
 
     if (eol || strncmp(p, header_end, header_end_size) != 0) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         SAFE_FREE(iv);
         return NULL;
     }
 
     if (ssh_buffer_add_data(buffer, "\0", 1) < 0) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         SAFE_FREE(iv);
         return NULL;
     }
 
     out = base64_to_bin(ssh_buffer_get(buffer));
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (out == NULL) {
         SAFE_FREE(iv);
         return NULL;
@@ -530,7 +530,7 @@ static ssh_buffer privatekey_string_to_buffer(const char *pkey, int type,
     if (algo) {
         if (privatekey_decrypt(algo, mode, key_len, iv, iv_len, out,
                     cb, userdata, desc) < 0) {
-            ssh_buffer_free(out);
+            SSH_BUFFER_FREE(out);
             SAFE_FREE(iv);
             return NULL;
         }
@@ -561,20 +561,20 @@ static int b64decode_rsa_privatekey(const char *pkey, gcry_sexp_t *r,
   }
 
   if (!asn1_check_sequence(buffer)) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return 0;
   }
 
   v = asn1_get_int(buffer);
   if (v == NULL) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return 0;
   }
 
   data = ssh_string_data(v);
   if (ssh_string_len(v) != 1 || data[0] != 0) {
-    ssh_string_free(v);
-    ssh_buffer_free(buffer);
+    SSH_STRING_FREE(v);
+    SSH_BUFFER_FREE(buffer);
     return 0;
   }
 
@@ -587,7 +587,7 @@ static int b64decode_rsa_privatekey(const char *pkey, gcry_sexp_t *r,
   unused2 = asn1_get_int(buffer);
   u = asn1_get_int(buffer);
 
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   if (n == NULL || e == NULL || d == NULL || p == NULL || q == NULL ||
       unused1 == NULL || unused2 == NULL|| u == NULL) {
@@ -608,20 +608,20 @@ static int b64decode_rsa_privatekey(const char *pkey, gcry_sexp_t *r,
 
 error:
   ssh_string_burn(n);
-  ssh_string_free(n);
+  SSH_STRING_FREE(n);
   ssh_string_burn(e);
-  ssh_string_free(e);
+  SSH_STRING_FREE(e);
   ssh_string_burn(d);
-  ssh_string_free(d);
+  SSH_STRING_FREE(d);
   ssh_string_burn(p);
-  ssh_string_free(p);
+  SSH_STRING_FREE(p);
   ssh_string_burn(q);
-  ssh_string_free(q);
-  ssh_string_free(unused1);
-  ssh_string_free(unused2);
+  SSH_STRING_FREE(q);
+  SSH_STRING_FREE(unused1);
+  SSH_STRING_FREE(unused2);
   ssh_string_burn(u);
-  ssh_string_free(u);
-  ssh_string_free(v);
+  SSH_STRING_FREE(u);
+  SSH_STRING_FREE(v);
 
   return rc;
 }
@@ -644,20 +644,20 @@ static int b64decode_dsa_privatekey(const char *pkey, gcry_sexp_t *r, ssh_auth_c
   }
 
   if (!asn1_check_sequence(buffer)) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return 0;
   }
 
   v = asn1_get_int(buffer);
   if (v == NULL) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return 0;
   }
 
   data = ssh_string_data(v);
   if (ssh_string_len(v) != 1 || data[0] != 0) {
-    ssh_string_free(v);
-    ssh_buffer_free(buffer);
+    SSH_STRING_FREE(v);
+    SSH_BUFFER_FREE(buffer);
     return 0;
   }
 
@@ -666,7 +666,7 @@ static int b64decode_dsa_privatekey(const char *pkey, gcry_sexp_t *r, ssh_auth_c
   g = asn1_get_int(buffer);
   y = asn1_get_int(buffer);
   x = asn1_get_int(buffer);
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   if (p == NULL || q == NULL || g == NULL || y == NULL || x == NULL) {
     rc = 0;
@@ -685,16 +685,16 @@ static int b64decode_dsa_privatekey(const char *pkey, gcry_sexp_t *r, ssh_auth_c
 
 error:
   ssh_string_burn(p);
-  ssh_string_free(p);
+  SSH_STRING_FREE(p);
   ssh_string_burn(q);
-  ssh_string_free(q);
+  SSH_STRING_FREE(q);
   ssh_string_burn(g);
-  ssh_string_free(g);
+  SSH_STRING_FREE(g);
   ssh_string_burn(y);
-  ssh_string_free(y);
+  SSH_STRING_FREE(y);
   ssh_string_burn(x);
-  ssh_string_free(x);
-  ssh_string_free(v);
+  SSH_STRING_FREE(x);
+  SSH_STRING_FREE(v);
 
   return rc;
 }
@@ -916,13 +916,13 @@ static int b64decode_ecdsa_privatekey(const char *pkey, gcry_sexp_t *r,
     }
 
  error:
-    ssh_buffer_free(buffer);
-    ssh_string_free(v);
+    SSH_BUFFER_FREE(buffer);
+    SSH_STRING_FREE(v);
     ssh_string_burn(d);
-    ssh_string_free(d);
-    ssh_string_free(oi);
+    SSH_STRING_FREE(d);
+    SSH_STRING_FREE(oi);
     ssh_string_burn(q);
-    ssh_string_free(q);
+    SSH_STRING_FREE(q);
 
     return valid;
 }
@@ -1560,7 +1560,7 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
     if (key->cert != NULL) {
         rc = ssh_buffer_add_buffer(buffer, key->cert);
         if (rc < 0) {
-            ssh_buffer_free(buffer);
+            SSH_BUFFER_FREE(buffer);
             return NULL;
         }
         goto makestring;
@@ -1568,14 +1568,14 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
 
     type_s = ssh_string_from_char(key->type_c);
     if (type_s == NULL) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         return NULL;
     }
 
     rc = ssh_buffer_add_ssh_string(buffer, type_s);
-    ssh_string_free(type_s);
+    SSH_STRING_FREE(type_s);
     if (rc < 0) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         return NULL;
     }
 
@@ -1631,13 +1631,13 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
             }
 
             ssh_string_burn(p);
-            ssh_string_free(p);
+            SSH_STRING_FREE(p);
             ssh_string_burn(g);
-            ssh_string_free(g);
+            SSH_STRING_FREE(g);
             ssh_string_burn(q);
-            ssh_string_free(q);
+            SSH_STRING_FREE(q);
             ssh_string_burn(n);
-            ssh_string_free(n);
+            SSH_STRING_FREE(n);
 
             break;
         case SSH_KEYTYPE_RSA:
@@ -1667,9 +1667,9 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
             }
 
             ssh_string_burn(e);
-            ssh_string_free(e);
+            SSH_STRING_FREE(e);
             ssh_string_burn(n);
-            ssh_string_free(n);
+            SSH_STRING_FREE(n);
 
             break;
         case SSH_KEYTYPE_ED25519:
@@ -1685,21 +1685,21 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
             type_s = ssh_string_from_char(
                        pki_key_ecdsa_nid_to_char(key->ecdsa_nid));
             if (type_s == NULL) {
-                ssh_buffer_free(buffer);
+                SSH_BUFFER_FREE(buffer);
                 return NULL;
             }
 
             rc = ssh_buffer_add_ssh_string(buffer, type_s);
-            ssh_string_free(type_s);
+            SSH_STRING_FREE(type_s);
             if (rc < 0) {
-                ssh_buffer_free(buffer);
+                SSH_BUFFER_FREE(buffer);
                 return NULL;
             }
 
             e = ssh_sexp_extract_mpi(key->ecdsa, "q", GCRYMPI_FMT_STD,
                                      GCRYMPI_FMT_STD);
             if (e == NULL) {
-                ssh_buffer_free(buffer);
+                SSH_BUFFER_FREE(buffer);
                 return NULL;
             }
 
@@ -1709,7 +1709,7 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
             }
 
             ssh_string_burn(e);
-            ssh_string_free(e);
+            SSH_STRING_FREE(e);
             e = NULL;
             break;
 #endif
@@ -1729,23 +1729,23 @@ makestring:
     if (rc < 0) {
         goto fail;
     }
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
 
     return str;
 fail:
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     ssh_string_burn(str);
-    ssh_string_free(str);
+    SSH_STRING_FREE(str);
     ssh_string_burn(e);
-    ssh_string_free(e);
+    SSH_STRING_FREE(e);
     ssh_string_burn(p);
-    ssh_string_free(p);
+    SSH_STRING_FREE(p);
     ssh_string_burn(g);
-    ssh_string_free(g);
+    SSH_STRING_FREE(g);
     ssh_string_burn(q);
-    ssh_string_free(q);
+    SSH_STRING_FREE(q);
     ssh_string_burn(n);
-    ssh_string_free(n);
+    SSH_STRING_FREE(n);
 
     return NULL;
 }
@@ -1832,7 +1832,7 @@ ssh_string pki_signature_to_blob(const ssh_signature sig)
             gcry_sexp_release(sexp);
             break;
         case SSH_KEYTYPE_ED25519:
-		sig_blob = pki_ed25519_sig_to_blob(sig);
+		sig_blob = pki_ed25519_signature_to_blob(sig);
 		break;
         case SSH_KEYTYPE_ECDSA_P256:
         case SSH_KEYTYPE_ECDSA_P384:
@@ -1852,40 +1852,40 @@ ssh_string pki_signature_to_blob(const ssh_signature sig)
                 R = ssh_sexp_extract_mpi(sig->ecdsa_sig, "r",
                                          GCRYMPI_FMT_USG, GCRYMPI_FMT_STD);
                 if (R == NULL) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     return NULL;
                 }
 
                 rc = ssh_buffer_add_ssh_string(b, R);
-                ssh_string_free(R);
+                SSH_STRING_FREE(R);
                 if (rc < 0) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     return NULL;
                 }
 
                 S = ssh_sexp_extract_mpi(sig->ecdsa_sig, "s",
                                          GCRYMPI_FMT_USG, GCRYMPI_FMT_STD);
                 if (S == NULL) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     return NULL;
                 }
 
                 rc = ssh_buffer_add_ssh_string(b, S);
-                ssh_string_free(S);
+                SSH_STRING_FREE(S);
                 if (rc < 0) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     return NULL;
                 }
 
                 sig_blob = ssh_string_new(ssh_buffer_get_len(b));
                 if (sig_blob == NULL) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     return NULL;
                 }
 
                 ssh_string_fill(sig_blob,
                                 ssh_buffer_get(b), ssh_buffer_get_len(b));
-                ssh_buffer_free(b);
+                SSH_BUFFER_FREE(b);
                 break;
             }
 #endif
@@ -1945,7 +1945,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
             SSH_LOG(SSH_LOG_DEBUG,
                     "DSA signature len: %lu",
                     (unsigned long)len);
-            ssh_print_hexa("DSA signature", ssh_string_data(sig_blob), len);
+            ssh_log_hexdump("DSA signature", ssh_string_data(sig_blob), len);
 #endif
 
             err = gcry_sexp_build(&sig->dsa_sig,
@@ -1980,7 +1980,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
 
 #ifdef DEBUG_CRYPTO
             SSH_LOG(SSH_LOG_DEBUG, "RSA signature len: %lu", (unsigned long)len);
-            ssh_print_hexa("RSA signature", ssh_string_data(sig_blob), len);
+            ssh_log_hexdump("RSA signature", ssh_string_data(sig_blob), len);
 #endif
 
             err = gcry_sexp_build(&sig->rsa_sig,
@@ -1994,7 +1994,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
             }
             break;
         case SSH_KEYTYPE_ED25519:
-		rc = pki_ed25519_sig_from_blob(sig, sig_blob);
+		rc = pki_signature_from_ed25519_blob(sig, sig_blob);
 		if (rc != SSH_OK){
 			ssh_signature_free(sig);
 			return NULL;
@@ -2019,24 +2019,24 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                                          ssh_string_data(sig_blob),
                                          ssh_string_len(sig_blob));
                 if (rc < 0) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     ssh_signature_free(sig);
                     return NULL;
                 }
 
                 r = ssh_buffer_get_ssh_string(b);
                 if (r == NULL) {
-                    ssh_buffer_free(b);
+                    SSH_BUFFER_FREE(b);
                     ssh_signature_free(sig);
                     return NULL;
                 }
 
                 s = ssh_buffer_get_ssh_string(b);
                 rlen = ssh_buffer_get_len(b);
-                ssh_buffer_free(b);
+                SSH_BUFFER_FREE(b);
                 if (s == NULL) {
                     ssh_string_burn(r);
-                    ssh_string_free(r);
+                    SSH_STRING_FREE(r);
                     ssh_signature_free(sig);
                     return NULL;
                 }
@@ -2047,16 +2047,16 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                             "sigblob: %lu",
                             (unsigned long)rlen);
                     ssh_string_burn(r);
-                    ssh_string_free(r);
+                    SSH_STRING_FREE(r);
                     ssh_string_burn(s);
-                    ssh_string_free(s);
+                    SSH_STRING_FREE(s);
                     ssh_signature_free(sig);
                     return NULL;
                 }
 
 #ifdef DEBUG_CRYPTO
-                ssh_print_hexa("r", ssh_string_data(r), ssh_string_len(r));
-                ssh_print_hexa("s", ssh_string_data(s), ssh_string_len(s));
+                ssh_log_hexdump("r", ssh_string_data(r), ssh_string_len(r));
+                ssh_log_hexdump("s", ssh_string_data(s), ssh_string_len(s));
 #endif
 
                 err = gcry_sexp_build(&sig->ecdsa_sig,
@@ -2067,9 +2067,9 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                                       ssh_string_len(s),
                                       ssh_string_data(s));
                 ssh_string_burn(r);
-                ssh_string_free(r);
+                SSH_STRING_FREE(r);
                 ssh_string_burn(s);
-                ssh_string_free(s);
+                SSH_STRING_FREE(s);
                 if (err) {
                     ssh_signature_free(sig);
                     return NULL;
@@ -2087,54 +2087,6 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
     return sig;
 }
 
-int pki_signature_verify(ssh_session session,
-                         const ssh_signature sig,
-                         const ssh_key key,
-                         const unsigned char *input,
-                         size_t input_len)
-{
-    int rc;
-
-    if (session == NULL || sig == NULL || key == NULL || input == NULL) {
-        SSH_LOG(SSH_LOG_TRACE, "Bad parameter provided to "
-                               "pki_signature_verify()");
-        return SSH_ERROR;
-    }
-
-    if (ssh_key_type_plain(key->type) != sig->type) {
-        SSH_LOG(SSH_LOG_WARN,
-                "Can not verify %s signature with %s key",
-                sig->type_c,
-                key->type_c);
-        return SSH_ERROR;
-    }
-
-    /* Check if public key and hash type are compatible */
-    rc = pki_key_check_hash_compatible(key, sig->hash_type);
-    if (rc != SSH_OK) {
-        return SSH_ERROR;
-    }
-
-    /* For ed25519 keys, verify using the input directly */
-    if (key->type == SSH_KEYTYPE_ED25519 ||
-        key->type == SSH_KEYTYPE_ED25519_CERT01)
-    {
-        rc = pki_ed25519_verify(key, sig, input, input_len);
-    } else {
-        /* For the other key types, calculate the hash and verify the signature */
-        rc = pki_verify_data_signature(sig, key, input, input_len);
-    }
-
-    if (rc != SSH_OK){
-        ssh_set_error(session,
-                      SSH_FATAL,
-                      "Signature verification error");
-        return SSH_ERROR;
-    }
-
-    return SSH_OK;
-}
-
 ssh_signature pki_do_sign_hash(const ssh_key privkey,
                                const unsigned char *hash,
                                size_t hlen,
@@ -2268,6 +2220,7 @@ ssh_signature pki_sign_data(const ssh_key privkey,
                             size_t input_len)
 {
     unsigned char hash[SHA512_DIGEST_LEN] = {0};
+    const unsigned char *sign_input = NULL;
     uint32_t hlen = 0;
     int rc;
 
@@ -2287,27 +2240,38 @@ ssh_signature pki_sign_data(const ssh_key privkey,
     case SSH_DIGEST_SHA256:
         sha256(input, input_len, hash);
         hlen = SHA256_DIGEST_LEN;
+        sign_input = hash;
         break;
     case SSH_DIGEST_SHA384:
         sha384(input, input_len, hash);
         hlen = SHA384_DIGEST_LEN;
+        sign_input = hash;
         break;
     case SSH_DIGEST_SHA512:
         sha512(input, input_len, hash);
         hlen = SHA512_DIGEST_LEN;
+        sign_input = hash;
         break;
     case SSH_DIGEST_SHA1:
         sha1(input, input_len, hash);
         hlen = SHA_DIGEST_LEN;
+        sign_input = hash;
         break;
     case SSH_DIGEST_AUTO:
+        if (privkey->type == SSH_KEYTYPE_ED25519) {
+            /* SSH_DIGEST_AUTO should only be used with ed25519 */
+            sign_input = input;
+            hlen = input_len;
+            break;
+        }
+        FALL_THROUGH;
     default:
         SSH_LOG(SSH_LOG_TRACE, "Unknown hash algorithm for type: %d",
                 hash_type);
         return NULL;
     }
 
-    return pki_do_sign_hash(privkey, hash, hlen, hash_type);
+    return pki_do_sign_hash(privkey, sign_input, hlen, hash_type);
 }
 
 /**
@@ -2336,6 +2300,8 @@ int pki_verify_data_signature(ssh_signature signature,
     unsigned char *hash = ghash + 1;
     uint32_t hlen = 0;
 
+    const unsigned char *verify_input = NULL;
+
     int rc;
 
     if (pubkey == NULL || ssh_key_is_private(pubkey) || input == NULL ||
@@ -2357,23 +2323,35 @@ int pki_verify_data_signature(ssh_signature signature,
         sha256(input, input_len, hash);
         hlen = SHA256_DIGEST_LEN;
         hash_type = "sha256";
+        verify_input = hash;
         break;
     case SSH_DIGEST_SHA384:
         sha384(input, input_len, hash);
         hlen = SHA384_DIGEST_LEN;
         hash_type = "sha384";
+        verify_input = hash;
         break;
     case SSH_DIGEST_SHA512:
         sha512(input, input_len, hash);
         hlen = SHA512_DIGEST_LEN;
         hash_type = "sha512";
+        verify_input = hash;
         break;
     case SSH_DIGEST_SHA1:
         sha1(input, input_len, hash);
         hlen = SHA_DIGEST_LEN;
         hash_type = "sha1";
+        verify_input = hash;
         break;
     case SSH_DIGEST_AUTO:
+        if (pubkey->type == SSH_KEYTYPE_ED25519 ||
+            pubkey->type == SSH_KEYTYPE_ED25519_CERT01)
+        {
+            verify_input = input;
+            hlen = input_len;
+            break;
+        }
+        FALL_THROUGH;
     default:
         SSH_LOG(SSH_LOG_TRACE, "Unknown sig->hash_type: %d", signature->hash_type);
         return SSH_ERROR;
@@ -2461,6 +2439,14 @@ int pki_verify_data_signature(ssh_signature signature,
             }
             break;
 #endif
+        case SSH_KEYTYPE_ED25519:
+        case SSH_KEYTYPE_ED25519_CERT01:
+            rc = pki_ed25519_verify(pubkey, signature, verify_input, hlen);
+            if (rc != SSH_OK) {
+                SSH_LOG(SSH_LOG_TRACE, "ED25519 error: Signature invalid");
+                return SSH_ERROR;
+            }
+            break;
         case SSH_KEYTYPE_RSA1:
         case SSH_KEYTYPE_UNKNOWN:
         default:
