@@ -166,15 +166,22 @@ int server_set_kex(ssh_session session)
         return -1;
     }
 
-    for (i = 0; i < 10; i++) {
+    for (i = 0; i < SSH_KEX_METHODS; i++) {
         wanted = session->opts.wanted_methods[i];
-        if (wanted  == NULL) {
+        if (wanted == NULL) {
             if (ssh_fips_mode()) {
                 wanted = ssh_kex_get_fips_methods(i);
             } else {
                 wanted = ssh_kex_get_default_methods(i);
             }
         }
+        if (wanted == NULL) {
+            for (j = 0; j < i; j++) {
+                SAFE_FREE(server->methods[j]);
+            }
+            return -1;
+        }
+
         server->methods[i] = strdup(wanted);
         if (server->methods[i] == NULL) {
             for (j = 0; j < i; j++) {
@@ -195,7 +202,7 @@ int ssh_server_init_kex(ssh_session session) {
     }
 
     /* free any currently-set methods: server_set_kex will allocate new ones */
-    for (i = 0; i < 10 /* SSH_KEX_METHODS */; i++) {
+    for (i = 0; i < SSH_KEX_METHODS; i++) {
         SAFE_FREE(session->next_crypto->server_kex.methods[i]);
     }
 
@@ -264,7 +271,11 @@ SSH_PACKET_CALLBACK(ssh_packet_kexdh_init){
   return SSH_PACKET_NOT_USED;
 }
 
-int ssh_get_key_params(ssh_session session, ssh_key *privkey){
+int
+ssh_get_key_params(ssh_session session,
+                   ssh_key *privkey,
+                   enum ssh_digest_e *digest)
+{
     ssh_key pubkey;
     ssh_string pubkey_blob;
     int rc;
@@ -290,6 +301,7 @@ int ssh_get_key_params(ssh_session session, ssh_key *privkey){
         *privkey = NULL;
     }
 
+    *digest = session->srv.hostkey_digest;
     rc = ssh_pki_export_privkey_to_pubkey(*privkey, &pubkey);
     if (rc < 0) {
       ssh_set_error(session, SSH_FATAL,
@@ -306,7 +318,7 @@ int ssh_get_key_params(ssh_session session, ssh_key *privkey){
     }
 
     rc = ssh_dh_import_next_pubkey_blob(session, pubkey_blob);
-    ssh_string_free(pubkey_blob);
+    SSH_STRING_FREE(pubkey_blob);
     if (rc != 0) {
         ssh_set_error(session,
                       SSH_FATAL,
@@ -505,9 +517,11 @@ static int ssh_server_kex_termination(void *s){
     return 1;
 }
 
-void ssh_set_auth_methods(ssh_session session, int auth_methods){
-	/* accept only methods in range */
-    session->auth.supported_methods = auth_methods & 0x3f;
+/* FIXME: auth_methods should be unsigned */
+void ssh_set_auth_methods(ssh_session session, int auth_methods)
+{
+    /* accept only methods in range */
+    session->auth.supported_methods = (uint32_t)auth_methods & 0x3fU;
 }
 
 /* Do the banner and key exchange */
@@ -815,12 +829,17 @@ int ssh_message_auth_kbdint_is_response(ssh_message msg) {
   return msg->auth_request.kbdint_response != 0;
 }
 
+/* FIXME: methods should be unsigned */
 int ssh_message_auth_set_methods(ssh_message msg, int methods) {
   if (msg == NULL || msg->session == NULL) {
     return -1;
   }
 
-  msg->session->auth.supported_methods = methods;
+  if (methods < 0) {
+      return -1;
+  }
+
+  msg->session->auth.supported_methods = (uint32_t)methods;
 
   return 0;
 }
@@ -1013,14 +1032,14 @@ int ssh_message_auth_reply_pk_ok_simple(ssh_message msg) {
 
     ret = ssh_pki_export_pubkey_blob(msg->auth_request.pubkey, &pubkey_blob);
     if (ret < 0) {
-        ssh_string_free(algo);
+        SSH_STRING_FREE(algo);
         return SSH_ERROR;
     }
 
     ret = ssh_message_auth_reply_pk_ok(msg, algo, pubkey_blob);
 
-    ssh_string_free(algo);
-    ssh_string_free(pubkey_blob);
+    SSH_STRING_FREE(algo);
+    SSH_STRING_FREE(pubkey_blob);
 
     return ret;
 }
