@@ -44,6 +44,7 @@
 #include "libssh/priv.h"
 #include "libssh/ssh2.h"
 #include "libssh/sftp.h"
+#include "libssh/sftp_priv.h"
 #include "libssh/buffer.h"
 #include "libssh/channels.h"
 #include "libssh/session.h"
@@ -57,7 +58,7 @@
 #define SFTP_BUFFER_SIZE_MAX 16384
 
 struct sftp_ext_struct {
-  unsigned int count;
+  uint32_t count;
   char **name;
   char **data;
 };
@@ -164,7 +165,7 @@ error:
     }
     if (sftp->read_packet != NULL) {
         if (sftp->read_packet->payload != NULL) {
-            ssh_buffer_free(sftp->read_packet->payload);
+            SSH_BUFFER_FREE(sftp->read_packet->payload);
         }
         SAFE_FREE(sftp->read_packet);
     }
@@ -229,7 +230,7 @@ error:
   ssh_set_error_oom(session);
   if (sftp->read_packet != NULL) {
     if (sftp->read_packet->payload != NULL) {
-      ssh_buffer_free(sftp->read_packet->payload);
+      SSH_BUFFER_FREE(sftp->read_packet->payload);
     }
     SAFE_FREE(sftp->read_packet);
   }
@@ -262,7 +263,7 @@ int sftp_server_init(sftp_session sftp){
   ssh_buffer_get_u32(packet->payload, &version);
   version = ntohl(version);
   SSH_LOG(SSH_LOG_PACKET, "Client version: %d", version);
-  sftp->client_version = version;
+  sftp->client_version = (int)version;
 
   reply = ssh_buffer_new();
   if (reply == NULL) {
@@ -278,22 +279,22 @@ int sftp_server_init(sftp_session sftp){
                       "1");
   if (rc != SSH_OK) {
     ssh_set_error_oom(session);
-    ssh_buffer_free(reply);
+    SSH_BUFFER_FREE(reply);
     return -1;
   }
 
   if (sftp_packet_write(sftp, SSH_FXP_VERSION, reply) < 0) {
-    ssh_buffer_free(reply);
+    SSH_BUFFER_FREE(reply);
     return -1;
   }
-  ssh_buffer_free(reply);
+  SSH_BUFFER_FREE(reply);
 
   SSH_LOG(SSH_LOG_PROTOCOL, "Server version sent");
 
   if (version > LIBSFTP_VERSION) {
     sftp->version = LIBSFTP_VERSION;
   } else {
-    sftp->version=version;
+    sftp->version = (int)version;
   }
 
   return 0;
@@ -358,11 +359,11 @@ void sftp_free(sftp_session sftp)
     SAFE_FREE(sftp);
 }
 
-int sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
+ssize_t sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
 {
     uint8_t header[5] = {0};
     uint32_t payload_size;
-    int size;
+    ssize_t size;
     int rc;
 
     /* Add size of type */
@@ -387,7 +388,7 @@ int sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
 
     if ((uint32_t)size != ssh_buffer_get_len(payload)) {
         SSH_LOG(SSH_LOG_PACKET,
-                "Had to write %d bytes, wrote only %d",
+                "Had to write %d bytes, wrote only %zd",
                 ssh_buffer_get_len(payload),
                 size);
     }
@@ -623,12 +624,13 @@ static int sftp_read_and_dispatch(sftp_session sftp)
     return 0;
 }
 
-void sftp_packet_free(sftp_packet packet) {
+void sftp_packet_free(sftp_packet packet)
+{
   if (packet == NULL) {
     return;
   }
 
-  ssh_buffer_free(packet->payload);
+  SSH_BUFFER_FREE(packet->payload);
   free(packet);
 }
 
@@ -651,15 +653,15 @@ int sftp_init(sftp_session sftp) {
   rc = ssh_buffer_pack(buffer, "d", LIBSFTP_VERSION);
   if (rc != SSH_OK) {
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
   if (sftp_packet_write(sftp, SSH_FXP_INIT, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   packet = sftp_packet_read(sftp);
   if (packet == NULL) {
@@ -683,7 +685,7 @@ int sftp_init(sftp_session sftp) {
       version);
   rc = ssh_buffer_unpack(packet->payload, "s", &ext_name);
   while (rc == SSH_OK) {
-    int count = sftp->ext->count;
+    uint32_t count = sftp->ext->count;
     char **tmp;
 
     rc = ssh_buffer_unpack(packet->payload, "s", &ext_data);
@@ -723,7 +725,7 @@ int sftp_init(sftp_session sftp) {
     rc = ssh_buffer_unpack(packet->payload, "s", &ext_name);
   }
 
-  sftp->version = sftp->server_version = version;
+  sftp->version = sftp->server_version = (int)version;
 
 
   return 0;
@@ -771,7 +773,7 @@ const char *sftp_extensions_get_data(sftp_session sftp, unsigned int idx) {
 
 int sftp_extension_supported(sftp_session sftp, const char *name,
     const char *data) {
-  int i, n;
+  size_t i, n;
 
   if (sftp == NULL || name == NULL || data == NULL) {
     return 0;
@@ -1014,13 +1016,13 @@ sftp_dir sftp_opendir(sftp_session sftp, const char *path)
                          path);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(payload);
+        SSH_BUFFER_FREE(payload);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_OPENDIR, payload);
-    ssh_buffer_free(payload);
+    SSH_BUFFER_FREE(payload);
     if (rc < 0) {
         return NULL;
     }
@@ -1121,7 +1123,7 @@ static sftp_attributes sftp_parse_attr_4(sftp_session sftp, ssh_buffer buf,
         break;
       }
       attr->owner = ssh_string_to_char(owner);
-      ssh_string_free(owner);
+      SSH_STRING_FREE(owner);
       if (attr->owner == NULL) {
         break;
       }
@@ -1131,7 +1133,7 @@ static sftp_attributes sftp_parse_attr_4(sftp_session sftp, ssh_buffer buf,
         break;
       }
       attr->group = ssh_string_to_char(group);
-      ssh_string_free(group);
+      SSH_STRING_FREE(group);
       if (attr->group == NULL) {
         break;
       }
@@ -1235,9 +1237,9 @@ static sftp_attributes sftp_parse_attr_4(sftp_session sftp, ssh_buffer buf,
 
   if (ok == 0) {
     /* break issued somewhere */
-    ssh_string_free(attr->acl);
-    ssh_string_free(attr->extended_type);
-    ssh_string_free(attr->extended_data);
+    SSH_STRING_FREE(attr->acl);
+    SSH_STRING_FREE(attr->extended_type);
+    SSH_STRING_FREE(attr->extended_data);
     SAFE_FREE(attr->owner);
     SAFE_FREE(attr->group);
     SAFE_FREE(attr);
@@ -1435,8 +1437,8 @@ static sftp_attributes sftp_parse_attr_3(sftp_session sftp, ssh_buffer buf,
     return attr;
 
     error:
-    ssh_string_free(attr->extended_type);
-    ssh_string_free(attr->extended_data);
+    SSH_STRING_FREE(attr->extended_type);
+    SSH_STRING_FREE(attr->extended_data);
     SAFE_FREE(attr->name);
     SAFE_FREE(attr->longname);
     SAFE_FREE(attr->owner);
@@ -1448,8 +1450,8 @@ static sftp_attributes sftp_parse_attr_3(sftp_session sftp, ssh_buffer buf,
     return NULL;
 }
 
-/* FIXME is this really needed as a public function? */
-int buffer_add_attributes(ssh_buffer buffer, sftp_attributes attr) {
+int buffer_add_attributes(ssh_buffer buffer, sftp_attributes attr)
+{
   uint32_t flags = (attr ? attr->flags : 0);
   int rc;
 
@@ -1494,8 +1496,10 @@ int buffer_add_attributes(ssh_buffer buffer, sftp_attributes attr) {
 }
 
 
-sftp_attributes sftp_parse_attr(sftp_session session, ssh_buffer buf,
-    int expectname) {
+sftp_attributes sftp_parse_attr(sftp_session session,
+                                ssh_buffer buf,
+                                int expectname)
+{
   switch(session->version) {
     case 4:
       return sftp_parse_attr_4(session, buf, expectname);
@@ -1545,12 +1549,12 @@ sftp_attributes sftp_readdir(sftp_session sftp, sftp_dir dir)
         if (rc != 0) {
             ssh_set_error_oom(sftp->session);
             sftp_set_error(sftp, SSH_FX_FAILURE);
-            ssh_buffer_free(payload);
+            SSH_BUFFER_FREE(payload);
             return NULL;
         }
 
         rc = sftp_packet_write(sftp, SSH_FXP_READDIR, payload);
-        ssh_buffer_free(payload);
+        SSH_BUFFER_FREE(payload);
         if (rc < 0) {
             return NULL;
         }
@@ -1624,7 +1628,7 @@ sftp_attributes sftp_readdir(sftp_session sftp, sftp_dir dir)
 
     dir->count--;
     if (dir->count == 0) {
-        ssh_buffer_free(dir->buffer);
+        SSH_BUFFER_FREE(dir->buffer);
         dir->buffer = NULL;
     }
 
@@ -1642,9 +1646,9 @@ void sftp_attributes_free(sftp_attributes file){
     return;
   }
 
-  ssh_string_free(file->acl);
-  ssh_string_free(file->extended_data);
-  ssh_string_free(file->extended_type);
+  SSH_STRING_FREE(file->acl);
+  SSH_STRING_FREE(file->extended_data);
+  SSH_STRING_FREE(file->extended_type);
 
   SAFE_FREE(file->name);
   SAFE_FREE(file->longname);
@@ -1677,13 +1681,13 @@ static int sftp_handle_close(sftp_session sftp, ssh_string handle)
                          handle);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return -1;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_CLOSE, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return -1;
     }
@@ -1733,7 +1737,7 @@ int sftp_close(sftp_file file){
   SAFE_FREE(file->name);
   if (file->handle){
     err = sftp_handle_close(file->sftp,file->handle);
-    ssh_string_free(file->handle);
+    SSH_STRING_FREE(file->handle);
   }
   /* FIXME: check server response and implement errno */
   SAFE_FREE(file);
@@ -1748,10 +1752,10 @@ int sftp_closedir(sftp_dir dir){
   SAFE_FREE(dir->name);
   if (dir->handle) {
     err = sftp_handle_close(dir->sftp, dir->handle);
-    ssh_string_free(dir->handle);
+    SSH_STRING_FREE(dir->handle);
   }
   /* FIXME: check server response and implement errno */
-  ssh_buffer_free(dir->buffer);
+  SSH_BUFFER_FREE(dir->buffer);
   SAFE_FREE(dir);
 
   return err;
@@ -1809,7 +1813,7 @@ sftp_file sftp_open(sftp_session sftp,
                          sftp_flags);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
@@ -1817,13 +1821,13 @@ sftp_file sftp_open(sftp_session sftp,
     rc = buffer_add_attributes(buffer, &attr);
     if (rc < 0) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_OPEN, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
@@ -1851,6 +1855,9 @@ sftp_file sftp_open(sftp_session sftp,
             return NULL;
         case SSH_FXP_HANDLE:
             handle = parse_handle_msg(msg);
+            if (handle == NULL) {
+                return NULL;
+            }
             sftp_message_free(msg);
             if ((flags & O_APPEND) == O_APPEND) {
                 stat_data = sftp_stat(sftp, file);
@@ -1895,7 +1902,7 @@ ssize_t sftp_read(sftp_file handle, void *buf, size_t count) {
   ssh_string datastring;
   size_t datalen;
   ssh_buffer buffer;
-  int id;
+  uint32_t id;
   int rc;
 
   if (handle->eof) {
@@ -1918,15 +1925,15 @@ ssize_t sftp_read(sftp_file handle, void *buf, size_t count) {
                        count);
   if (rc != SSH_OK){
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
   if (sftp_packet_write(handle->sftp, SSH_FXP_READ, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   while (msg == NULL) {
     if (handle->nonblocking) {
@@ -1977,12 +1984,12 @@ ssize_t sftp_read(sftp_file handle, void *buf, size_t count) {
             "Received a too big DATA packet from sftp server: "
             "%" PRIdS " and asked for %" PRIdS,
             datalen, count);
-        ssh_string_free(datastring);
+        SSH_STRING_FREE(datastring);
         return -1;
       }
       handle->offset += (uint64_t)datalen;
       memcpy(buf, ssh_string_data(datastring), datalen);
-      ssh_string_free(datastring);
+      SSH_STRING_FREE(datastring);
       return datalen;
     default:
       ssh_set_error(sftp->session, SSH_FATAL,
@@ -2019,15 +2026,15 @@ int sftp_async_read_begin(sftp_file file, uint32_t len){
                        len);
   if (rc != SSH_OK) {
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
   if (sftp_packet_write(sftp, SSH_FXP_READ, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   file->offset += len; /* assume we'll read len bytes */
 
@@ -2099,14 +2106,14 @@ int sftp_async_read(sftp_file file, void *data, uint32_t size, uint32_t id){
             "Received a too big DATA packet from sftp server: "
             "%" PRIdS " and asked for %u",
             ssh_string_len(datastring), size);
-        ssh_string_free(datastring);
+        SSH_STRING_FREE(datastring);
         return SSH_ERROR;
       }
       len = ssh_string_len(datastring);
       /* Update the offset with the correct value */
       file->offset = file->offset - (size - len);
       memcpy(data, ssh_string_data(datastring), len);
-      ssh_string_free(datastring);
+      SSH_STRING_FREE(datastring);
       return len;
     default:
       ssh_set_error(sftp->session,SSH_FATAL,"Received message %d during read!",msg->packet_type);
@@ -2124,8 +2131,8 @@ ssize_t sftp_write(sftp_file file, const void *buf, size_t count) {
   sftp_status_message status;
   ssh_buffer buffer;
   uint32_t id;
-  int len;
-  int packetlen;
+  ssize_t len;
+  size_t packetlen;
   int rc;
 
   buffer = ssh_buffer_new();
@@ -2146,16 +2153,16 @@ ssize_t sftp_write(sftp_file file, const void *buf, size_t count) {
                        (size_t)count, buf);
   if (rc != SSH_OK){
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
   packetlen=ssh_buffer_get_len(buffer);
   len = sftp_packet_write(file->sftp, SSH_FXP_WRITE, buffer);
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
   if (len < 0) {
     return -1;
-  } else  if (len != packetlen) {
+  } else  if ((size_t)len != packetlen) {
     SSH_LOG(SSH_LOG_PACKET,
         "Could not write as much data as expected");
   }
@@ -2261,16 +2268,16 @@ int sftp_unlink(sftp_session sftp, const char *file) {
                        file);
   if (rc != SSH_OK) {
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
 
   if (sftp_packet_write(sftp, SSH_FXP_REMOVE, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   while (msg == NULL) {
     if (sftp_read_and_dispatch(sftp)) {
@@ -2336,15 +2343,15 @@ int sftp_rmdir(sftp_session sftp, const char *directory) {
                        directory);
   if (rc != SSH_OK) {
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
   if (sftp_packet_write(sftp, SSH_FXP_RMDIR, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   while (msg == NULL) {
     if (sftp_read_and_dispatch(sftp) < 0) {
@@ -2413,7 +2420,7 @@ int sftp_mkdir(sftp_session sftp, const char *directory, mode_t mode)
                          directory);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return -1;
     }
@@ -2421,13 +2428,13 @@ int sftp_mkdir(sftp_session sftp, const char *directory, mode_t mode)
     rc = buffer_add_attributes(buffer, &attr);
     if (rc < 0) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return -1;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_MKDIR, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return -1;
     }
@@ -2509,7 +2516,7 @@ int sftp_rename(sftp_session sftp, const char *original, const char *newname) {
                        newname);
   if (rc != SSH_OK) {
     ssh_set_error_oom(sftp->session);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
@@ -2521,10 +2528,10 @@ int sftp_rename(sftp_session sftp, const char *original, const char *newname) {
   }
 
   if (sftp_packet_write(sftp, SSH_FXP_RENAME, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   while (msg == NULL) {
     if (sftp_read_and_dispatch(sftp) < 0) {
@@ -2592,7 +2599,7 @@ int sftp_setstat(sftp_session sftp, const char *file, sftp_attributes attr)
                          file);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return -1;
     }
@@ -2600,13 +2607,13 @@ int sftp_setstat(sftp_session sftp, const char *file, sftp_attributes attr)
     rc = buffer_add_attributes(buffer, attr);
     if (rc != 0) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return -1;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_SETSTAT, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return -1;
     }
@@ -2732,16 +2739,16 @@ int sftp_symlink(sftp_session sftp, const char *target, const char *dest) {
   }
   if (rc != SSH_OK){
       ssh_set_error_oom(sftp->session);
-      ssh_buffer_free(buffer);
+      SSH_BUFFER_FREE(buffer);
       sftp_set_error(sftp, SSH_FX_FAILURE);
       return -1;
   }
 
   if (sftp_packet_write(sftp, SSH_FXP_SYMLINK, buffer) < 0) {
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     return -1;
   }
-  ssh_buffer_free(buffer);
+  SSH_BUFFER_FREE(buffer);
 
   while (msg == NULL) {
     if (sftp_read_and_dispatch(sftp) < 0) {
@@ -2820,13 +2827,13 @@ char *sftp_readlink(sftp_session sftp, const char *path)
                          path);
     if (rc < 0) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_READLINK, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
@@ -2947,13 +2954,13 @@ sftp_statvfs_t sftp_statvfs(sftp_session sftp, const char *path)
                          path);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_EXTENDED, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
@@ -3087,7 +3094,7 @@ int sftp_fsync(sftp_file file)
 
     rc = -1;
 done:
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
 
     return rc;
 }
@@ -3122,13 +3129,13 @@ sftp_statvfs_t sftp_fstatvfs(sftp_file file)
                          file->handle);
     if (rc < 0) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_EXTENDED, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
@@ -3208,13 +3215,13 @@ char *sftp_canonicalize_path(sftp_session sftp, const char *path)
                          path);
     if (rc < 0) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, SSH_FXP_REALPATH, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
@@ -3299,13 +3306,13 @@ static sftp_attributes sftp_xstat(sftp_session sftp,
                          path);
     if (rc != SSH_OK) {
         ssh_set_error_oom(sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(sftp, param, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
@@ -3377,13 +3384,13 @@ sftp_attributes sftp_fstat(sftp_file file)
                          file->handle);
     if (rc != SSH_OK) {
         ssh_set_error_oom(file->sftp->session);
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         sftp_set_error(file->sftp, SSH_FX_FAILURE);
         return NULL;
     }
 
     rc = sftp_packet_write(file->sftp, SSH_FXP_FSTAT, buffer);
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc < 0) {
         return NULL;
     }
