@@ -37,7 +37,7 @@
 
 static int sshd_setup(void **state)
 {
-    torture_setup_sshd_server(state);
+    torture_setup_sshd_server(state, true);
 
     return 0;
 }
@@ -52,12 +52,24 @@ static int session_setup(void **state)
 {
     struct torture_state *s = *state;
     int verbosity = torture_libssh_verbosity();
+    struct passwd *pwd;
+    bool b = false;
+    int rc;
+
+    pwd = getpwnam("bob");
+    assert_non_null(pwd);
+
+    rc = setuid(pwd->pw_uid);
+    assert_return_code(rc, errno);
 
     s->ssh.session = ssh_new();
     assert_non_null(s->ssh.session);
 
     ssh_options_set(s->ssh.session, SSH_OPTIONS_LOG_VERBOSITY, &verbosity);
     ssh_options_set(s->ssh.session, SSH_OPTIONS_HOST, TORTURE_SSH_SERVER);
+    /* Make sure no other configuration options from system will get used */
+    rc = ssh_options_set(s->ssh.session, SSH_OPTIONS_PROCESS_CONFIG, &b);
+    assert_ssh_return_code(s->ssh.session, rc);
 
     return 0;
 }
@@ -75,19 +87,12 @@ static int session_teardown(void **state)
 static int pubkey_setup(void **state)
 {
     int rc;
-    struct passwd *pwd;
 
     rc = session_setup(state);
     if (rc != 0) {
         return rc;
     }
 
-    pwd = getpwnam("bob");
-    assert_non_null(pwd);
-
-    rc = setuid(pwd->pw_uid);
-    assert_return_code(rc, errno);
-
     /* Make sure we do not interfere with another ssh-agent */
     unsetenv("SSH_AUTH_SOCK");
     unsetenv("SSH_AGENT_PID");
@@ -538,13 +543,55 @@ static void torture_auth_cert(void **state) {
     SSH_KEY_FREE(cert);
 }
 
-static void torture_auth_agent_cert(void **state) {
-  /* Setup loads a different key, tests are exactly the same. */
-  torture_auth_agent(state);
+static void torture_auth_agent_cert(void **state)
+{
+    struct torture_state *s = *state;
+    ssh_session session = s->ssh.session;
+    int rc;
+
+    /* Skip this test if in FIPS mode.
+     *
+     * OpenSSH agent has a bug which makes it to not use SHA2 in signatures when
+     * using certificates. It always uses SHA1.
+     *
+     * This should be removed as soon as OpenSSH agent bug is fixed.
+     * (see https://gitlab.com/libssh/libssh-mirror/merge_requests/34) */
+    if (ssh_fips_mode()) {
+        skip();
+    } else {
+        /* After the bug is solved, this also should be removed */
+        rc = ssh_options_set(session, SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES,
+                             "ssh-rsa-cert-v01@openssh.com");
+        assert_int_equal(rc, SSH_OK);
+    }
+
+    /* Setup loads a different key, tests are exactly the same. */
+    torture_auth_agent(state);
 }
 
-static void torture_auth_agent_cert_nonblocking(void **state) {
-  torture_auth_agent_nonblocking(state);
+static void torture_auth_agent_cert_nonblocking(void **state)
+{
+    struct torture_state *s = *state;
+    ssh_session session = s->ssh.session;
+    int rc;
+
+    /* Skip this test if in FIPS mode.
+     *
+     * OpenSSH agent has a bug which makes it to not use SHA2 in signatures when
+     * using certificates. It always uses SHA1.
+     *
+     * This should be removed as soon as OpenSSH agent bug is fixed.
+     * (see https://gitlab.com/libssh/libssh-mirror/merge_requests/34) */
+    if (ssh_fips_mode()) {
+        skip();
+    } else {
+        /* After the bug is solved, this also should be removed */
+        rc = ssh_options_set(session, SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES,
+                             "ssh-rsa-cert-v01@openssh.com");
+        assert_int_equal(rc, SSH_OK);
+    }
+
+    torture_auth_agent_nonblocking(state);
 }
 
 static void torture_auth_pubkey_types(void **state)
@@ -569,7 +616,7 @@ static void torture_auth_pubkey_types(void **state)
 
     /* Disable RSA key types for authentication */
     rc = ssh_options_set(session, SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES,
-                         "ssh-dss");
+                         "ecdsa-sha2-nistp384");
     assert_ssh_return_code(session, rc);
 
     rc = ssh_userauth_publickey_auto(session, NULL, NULL);
@@ -626,8 +673,23 @@ static void torture_auth_pubkey_types_ed25519(void **state)
 {
     struct torture_state *s = *state;
     ssh_session session = s->ssh.session;
+    char bob_ssh_key[1024];
+    ssh_key privkey = NULL;
+    struct passwd *pwd;
     int rc;
 
+    if (ssh_fips_mode()) {
+        skip();
+    }
+
+    pwd = getpwnam("bob");
+    assert_non_null(pwd);
+
+    snprintf(bob_ssh_key,
+             sizeof(bob_ssh_key),
+             "%s/.ssh/id_ed25519",
+             pwd->pw_dir);
+
     rc = ssh_options_set(session, SSH_OPTIONS_USER, TORTURE_SSH_USER_ALICE);
     assert_ssh_return_code(session, rc);
 
@@ -642,12 +704,16 @@ static void torture_auth_pubkey_types_ed25519(void **state)
     rc = ssh_userauth_list(session, NULL);
     assert_true(rc & SSH_AUTH_METHOD_PUBLICKEY);
 
-    /* Enable only DSA keys -- authentication should fail */
+    /* Import the ED25519 private key */
+    rc = ssh_pki_import_privkey_file(bob_ssh_key, NULL, NULL, NULL, &privkey);
+    assert_int_equal(rc, SSH_OK);
+
+    /* Enable only RSA keys -- authentication should fail */
     rc = ssh_options_set(session, SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES,
-                         "ssh-dss");
+                         "ssh-rsa");
     assert_ssh_return_code(session, rc);
 
-    rc = ssh_userauth_publickey_auto(session, NULL, NULL);
+    rc = ssh_userauth_publickey(session, NULL, privkey);
     assert_int_equal(rc, SSH_AUTH_DENIED);
 
     /* Verify we can use also ed25519 keys */
@@ -655,9 +721,10 @@ static void torture_auth_pubkey_types_ed25519(void **state)
                          "ssh-ed25519");
     assert_ssh_return_code(session, rc);
 
-    rc = ssh_userauth_publickey_auto(session, NULL, NULL);
+    rc = ssh_userauth_publickey(session, NULL, privkey);
     assert_int_equal(rc, SSH_AUTH_SUCCESS);
 
+    SSH_KEY_FREE(privkey);
 }
 
 static void torture_auth_pubkey_types_nonblocking(void **state)
@@ -687,7 +754,7 @@ static void torture_auth_pubkey_types_nonblocking(void **state)
 
     /* Disable RSA key types for authentication */
     rc = ssh_options_set(session, SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES,
-                         "ssh-dss");
+                         "ecdsa-sha2-nistp521");
     assert_ssh_return_code(session, rc);
 
     do {
@@ -758,8 +825,23 @@ static void torture_auth_pubkey_types_ed25519_nonblocking(void **state)
 {
     struct torture_state *s = *state;
     ssh_session session = s->ssh.session;
+    char bob_ssh_key[1024];
+    ssh_key privkey = NULL;
+    struct passwd *pwd;
     int rc;
 
+    if (ssh_fips_mode()) {
+        skip();
+    }
+
+    pwd = getpwnam("bob");
+    assert_non_null(pwd);
+
+    snprintf(bob_ssh_key,
+             sizeof(bob_ssh_key),
+             "%s/.ssh/id_ed25519",
+             pwd->pw_dir);
+
     rc = ssh_options_set(session, SSH_OPTIONS_USER, TORTURE_SSH_USER_ALICE);
     assert_ssh_return_code(session, rc);
 
@@ -779,13 +861,17 @@ static void torture_auth_pubkey_types_ed25519_nonblocking(void **state)
     rc = ssh_userauth_list(session, NULL);
     assert_true(rc & SSH_AUTH_METHOD_PUBLICKEY);
 
-    /* Enable only DSA keys -- authentication should fail */
+    /* Import the ED25519 private key */
+    rc = ssh_pki_import_privkey_file(bob_ssh_key, NULL, NULL, NULL, &privkey);
+    assert_int_equal(rc, SSH_OK);
+
+    /* Enable only RSA keys -- authentication should fail */
     rc = ssh_options_set(session, SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES,
-                         "ssh-dss");
+                         "ssh-rsa");
     assert_ssh_return_code(session, rc);
 
     do {
-        rc = ssh_userauth_publickey_auto(session, NULL, NULL);
+        rc = ssh_userauth_publickey(session, NULL, privkey);
     } while (rc == SSH_AUTH_AGAIN);
     assert_int_equal(rc, SSH_AUTH_DENIED);
 
@@ -795,7 +881,7 @@ static void torture_auth_pubkey_types_ed25519_nonblocking(void **state)
     assert_ssh_return_code(session, rc);
 
     do {
-        rc = ssh_userauth_publickey_auto(session, NULL, NULL);
+        rc = ssh_userauth_publickey(session, NULL, privkey);
     } while (rc == SSH_AUTH_AGAIN);
     assert_int_equal(rc, SSH_AUTH_SUCCESS);
 
