@@ -23,6 +23,13 @@ struct pkd_daemon_args {
     enum pkd_hostkey_type_e type;
     const char *hostkeypath;
 
+    struct {
+        const uint8_t *buf;
+        size_t len;
+    } payload;
+
+    uint64_t rekey_data_limit;
+
     struct {
         int list;
 
