@@ -430,10 +430,7 @@ static size_t setup_hostkey_files(struct test_server_st *tss)
 
     hostkey_files[0] = tss->rsa_hostkey;
 
-#ifndef TEST_ALL_CRYPTO_COMBINATIONS
-    goto end;
-#endif
-
+#ifdef TEST_ALL_CRYPTO_COMBINATIONS
     hostkey_files[1] = tss->ecdsa_256_hostkey;
     hostkey_files[2] = tss->ecdsa_384_hostkey;
     hostkey_files[3] = tss->ecdsa_521_hostkey;
@@ -448,8 +445,8 @@ static size_t setup_hostkey_files(struct test_server_st *tss)
         num_hostkey_files++;
 #endif
     }
+#endif /* TEST_ALL_CRYPTO_COMBINATIONS */
 
-end:
     return num_hostkey_files;
 }
 
@@ -471,9 +468,6 @@ static void torture_server_config_hostkey(void **state)
                 "HostKey %s\n",
                 hostkey_files[i]);
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
     }
@@ -513,9 +507,6 @@ static void torture_server_config_ciphers(void **state)
                  "HostKey %s\nCiphers %s\n",
                  hostkey_files[i], ciphers);
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
 
@@ -527,9 +518,6 @@ static void torture_server_config_ciphers(void **state)
                     "HostKey %s\nCiphers %s\n",
                     hostkey_files[i], tokens->tokens[j]);
 
-            rc = try_config_content(state, config_content, true);
-            assert_int_equal(rc, 0);
-
             rc = try_config_content(state, config_content, false);
             assert_int_equal(rc, 0);
 
@@ -574,9 +562,6 @@ static void torture_server_config_macs(void **state)
                  "HostKey %s\nMACs %s\n",
                  hostkey_files[i], macs);
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
 
@@ -588,9 +573,6 @@ static void torture_server_config_macs(void **state)
                     "HostKey %s\nMACs %s\n",
                     hostkey_files[i], tokens->tokens[j]);
 
-            rc = try_config_content(state, config_content, true);
-            assert_int_equal(rc, 0);
-
             rc = try_config_content(state, config_content, false);
             assert_int_equal(rc, 0);
 
@@ -635,9 +617,6 @@ static void torture_server_config_kex(void **state)
                  "HostKey %s\nKexAlgorithms %s\n",
                  hostkey_files[i], kex);
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
 
@@ -649,9 +628,6 @@ static void torture_server_config_kex(void **state)
                     "HostKey %s\nKexAlgorithms %s\n",
                     hostkey_files[i], tokens->tokens[j]);
 
-            rc = try_config_content(state, config_content, true);
-            assert_int_equal(rc, 0);
-
             rc = try_config_content(state, config_content, false);
             assert_int_equal(rc, 0);
 
@@ -691,9 +667,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
                  "HostKey %s\nHostKeyAlgorithms %s\n",
                  hostkey_files[i], allowed);
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
     }
@@ -707,9 +680,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
                 "HostKey %s\nHostkeyAlgorithms %s\n",
                 tss->ed25519_hostkey, "ssh-ed25519");
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
 
@@ -719,9 +689,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
                 "HostKey %s\nHostkeyAlgorithms %s\n",
                 tss->rsa_hostkey, "ssh-rsa");
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
     }
@@ -732,9 +699,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
             "HostKey %s\nHostkeyAlgorithms %s\n",
             tss->rsa_hostkey, "rsa-sha2-256");
 
-    rc = try_config_content(state, config_content, true);
-    assert_int_equal(rc, 0);
-
     rc = try_config_content(state, config_content, false);
     assert_int_equal(rc, 0);
 
@@ -744,9 +708,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
             "HostKey %s\nHostkeyAlgorithms %s\n",
             tss->rsa_hostkey, "rsa-sha2-512");
 
-    rc = try_config_content(state, config_content, true);
-    assert_int_equal(rc, 0);
-
     rc = try_config_content(state, config_content, false);
     assert_int_equal(rc, 0);
 
@@ -756,9 +717,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
             "HostKey %s\nHostkeyAlgorithms %s\n",
             tss->ecdsa_256_hostkey, "ecdsa-sha2-nistp256");
 
-    rc = try_config_content(state, config_content, true);
-    assert_int_equal(rc, 0);
-
     rc = try_config_content(state, config_content, false);
     assert_int_equal(rc, 0);
 
@@ -768,9 +726,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
             "HostKey %s\nHostkeyAlgorithms %s\n",
             tss->ecdsa_384_hostkey, "ecdsa-sha2-nistp384");
 
-    rc = try_config_content(state, config_content, true);
-    assert_int_equal(rc, 0);
-
     rc = try_config_content(state, config_content, false);
     assert_int_equal(rc, 0);
 
@@ -780,9 +735,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
             "HostKey %s\nHostkeyAlgorithms %s\n",
             tss->ecdsa_521_hostkey, "ecdsa-sha2-nistp521");
 
-    rc = try_config_content(state, config_content, true);
-    assert_int_equal(rc, 0);
-
     rc = try_config_content(state, config_content, false);
     assert_int_equal(rc, 0);
 
@@ -794,9 +746,6 @@ static void torture_server_config_hostkey_algorithms(void **state)
                 "HostKey %s\nHostkeyAlgorithms %s\n",
                 tss->dsa_hostkey, "ssh-dss");
 
-        rc = try_config_content(state, config_content, true);
-        assert_int_equal(rc, 0);
-
         rc = try_config_content(state, config_content, false);
         assert_int_equal(rc, 0);
     }
@@ -818,9 +767,6 @@ static void torture_server_config_unknown(void **state)
             "HostKey %s\nUnknownOption unknown-value1,unknown-value2\n",
             tss->rsa_hostkey);
 
-    rc = try_config_content(state, config_content, true);
-    assert_int_equal(rc, 0);
-
     rc = try_config_content(state, config_content, false);
     assert_int_equal(rc, 0);
 }
